@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Spam Buttons Plugin
-  * Copyright (c) 2005-2007 Paul Lesniewski <paul@squirrelmail.org>,
+  * Copyright (c) 2005-2008 Paul Lesniewski <paul@squirrelmail.org>,
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -11,8 +11,6 @@
   */
 
 
-include_once(SM_PATH . 'functions/identity.php');
-
 
 //
 // ripped from src/compose.php
@@ -21,11 +19,10 @@ include_once(SM_PATH . 'functions/identity.php');
 /* This function is used when not sending or adding attachments */
 function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $session='') {
     global $editor_size, $default_use_priority, $body, $idents,
-        $use_signature, $data_dir, $username,
-        $key, $imapServerAddress, $imapPort, $compose_messages,
-        $composeMessage, $body_quote, $request_mdn, $request_dr,
-        $mdn_user_support, $languages, $squirrelmail_language,
-        $default_charset;
+        $use_signature, $composesession, $data_dir, $username,
+        $username, $key, $imapServerAddress, $imapPort, $compose_messages,
+        $composeMessage, $body_quote;
+    global $languages, $squirrelmail_language, $default_charset;
 
     /*
      * Set $default_charset to correspond with the user's selection
@@ -38,7 +35,7 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
     $mailprio = 3;
 
     if ($passed_id) {
-        $imapConnection = sqimap_login($username, false, $imapServerAddress,
+        $imapConnection = sqimap_login($username, $key, $imapServerAddress,
                 $imapPort, 0);
 
         sqimap_mailbox_select($imapConnection, $mailbox);
@@ -71,26 +68,24 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
             $orig_header = $message->rfc822_header;
         }
 
+        $encoding = $message->header->encoding;
         $type0 = $message->type0;
         $type1 = $message->type1;
         foreach ($entities as $ent) {
-            $msg = $message->getEntity($ent);
-            $type0 = $msg->type0;
-            $type1 = $msg->type1;
             $unencoded_bodypart = mime_fetch_body($imapConnection, $passed_id, $ent);
             $body_part_entity = $message->getEntity($ent);
             $bodypart = decodeBody($unencoded_bodypart,
                     $body_part_entity->header->encoding);
             if ($type1 == 'html') {
                 $bodypart = str_replace("\n", ' ', $bodypart);
-                $bodypart = preg_replace(array('/<\/?p>/i','/<div><\/div>/i','/<br\s*(\/)*>/i','/<\/?div>/i'), "\n", $bodypart);
+                $bodypart = preg_replace(array('/<p>/i','/<br\s*(\/)*>/i'), "\n", $bodypart);
                 $bodypart = str_replace(array('&nbsp;','&gt;','&lt;'),array(' ','>','<'),$bodypart);
                 $bodypart = strip_tags($bodypart);
             }
             if (isset($languages[$squirrelmail_language]['XTRA_CODE']) &&
-                    function_exists($languages[$squirrelmail_language]['XTRA_CODE'] . '_decode')) {
+                    function_exists($languages[$squirrelmail_language]['XTRA_CODE'])) {
                 if (mb_detect_encoding($bodypart) != 'ASCII') {
-                    $bodypart = call_user_func($languages[$squirrelmail_language]['XTRA_CODE'] . '_decode', $bodypart);
+                    $bodypart = $languages[$squirrelmail_language]['XTRA_CODE']('decode', $bodypart);
                 }
             }
 
@@ -117,6 +112,7 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
             $mailprio = '';
         }
 
+        $identity = '';
         $from_o = $orig_header->from;
         if (is_array($from_o)) {
             if (isset($from_o[0])) {
@@ -153,22 +149,22 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                 $send_to = decodeHeader($orig_header->getAddr_s('to'),false,false,true);
                 $send_to_cc = decodeHeader($orig_header->getAddr_s('cc'),false,false,true);
                 $send_to_bcc = decodeHeader($orig_header->getAddr_s('bcc'),false,false,true);
-                $send_from = $orig_header->getAddr_s('from');
-                $send_from_parts = new AddressStructure();
-                $send_from_parts = $orig_header->parseAddress($send_from);
-                $send_from_add = $send_from_parts->mailbox . '@' . $send_from_parts->host;
-                $identity = find_identity(array($send_from_add));
+                // FIXME: ident support?
                 $subject = decodeHeader($orig_header->subject,false,false,true);
-
-                // Remember the receipt settings
-                $request_mdn = $mdn_user_support && !empty($orig_header->dnt) ? '1' : '0';
-                $request_dr = $mdn_user_support && !empty($orig_header->drnt) ? '1' : '0';
-
                 /* remember the references and in-reply-to headers in case of an reply */
                 $composeMessage->rfc822_header->more_headers['References'] = $orig_header->references;
                 $composeMessage->rfc822_header->more_headers['In-Reply-To'] = $orig_header->in_reply_to;
-                // rewrap the body to clean up quotations and line lengths
-                sqBodyWrap($body, $editor_size);
+                $body_ary = explode("\n", $body);
+                $cnt = count($body_ary) ;
+                $body = '';
+                for ($i=0; $i < $cnt; $i++) {
+                    if (!ereg("^[>\\s]*$", $body_ary[$i])  || !$body_ary[$i]) {
+                        sqWordWrap($body_ary[$i], $editor_size, $default_charset );
+                        $body .= $body_ary[$i] . "\n";
+                    }
+                    unset($body_ary[$i]);
+                }
+                sqUnWordWrap($body);
                 $composeMessage = getAttachments($message, $composeMessage, $passed_id, $entities, $imapConnection);
                 break;
             case ('edit_as_new'):
@@ -179,25 +175,30 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                 $mailprio = $orig_header->priority;
                 $orig_from = '';
                 $composeMessage = getAttachments($message, $composeMessage, $passed_id, $entities, $imapConnection);
-                // rewrap the body to clean up quotations and line lengths
-                sqBodyWrap($body, $editor_size);
+                sqUnWordWrap($body);
                 break;
             case ('forward'):
                 $send_to = '';
-                $subject = getforwardSubject(decodeHeader($orig_header->subject,false,false,true));
+                $subject = decodeHeader($orig_header->subject,false,false,true);
+                if ((substr(strtolower($subject), 0, 4) != 'fwd:') &&
+                    (substr(strtolower($subject), 0, 5) != '[fwd:') &&
+                    (substr(strtolower($subject), 0, 6) != '[ fwd:')) {
+                    $subject = '[Fwd: ' . $subject . ']';
+                }
                 $body = getforwardHeader($orig_header) . $body;
-                // the logic for calling sqUnWordWrap here would be to allow the browser to wrap the lines
-                // forwarded message text should be as undisturbed as possible, so commenting out this call
-                // sqUnWordWrap($body);
                 $composeMessage = getAttachments($message, $composeMessage, $passed_id, $entities, $imapConnection);
-
-                //add a blank line after the forward headers
                 $body = "\n" . $body;
                 break;
             case ('forward_as_attachment'):
-                $subject = getforwardSubject(decodeHeader($orig_header->subject,false,false,true));
                 $composeMessage = getMessage_RFC822_Attachment($message, $composeMessage, $passed_id, $passed_ent_id, $imapConnection);
                 $body = '';
+/// CHANGE FOR SPAM_BUTTONS PLUGIN (added next 6 lines)
+                $subject = decodeHeader($orig_header->subject,false,false,true);
+                if ((substr(strtolower($subject), 0, 4) != 'fwd:') &&
+                    (substr(strtolower($subject), 0, 5) != '[fwd:') &&
+                    (substr(strtolower($subject), 0, 6) != '[ fwd:')) {
+                    $subject = '[Fwd: ' . $subject . ']';
+                }
                 break;
             case ('reply_all'):
                 if(isset($orig_header->mail_followup_to) && $orig_header->mail_followup_to) {
@@ -207,8 +208,7 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                     $send_to_cc = decodeHeader($send_to_cc,false,false,true);
                 }
             case ('reply'):
-                // skip this if send_to was already set right above here
-                if(!$send_to) {
+                if (!$send_to) {
                     $send_to = $orig_header->reply_to;
                     if (is_array($send_to) && count($send_to)) {
                         $send_to = $orig_header->getAddr_s('reply_to');
@@ -220,7 +220,6 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                 }
                 $send_to = decodeHeader($send_to,false,false,true);
                 $subject = decodeHeader($orig_header->subject,false,false,true);
-                $subject = str_replace('"', "'", $subject);
                 $subject = trim($subject);
                 if (substr(strtolower($subject), 0, 3) != 're:') {
                     $subject = 'Re: ' . $subject;
@@ -228,23 +227,20 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
                 /* this corrects some wrapping/quoting problems on replies */
                 $rewrap_body = explode("\n", $body);
                 $from = (is_array($orig_header->from) && !empty($orig_header->from)) ? $orig_header->from[0] : $orig_header->from;
+                sqUnWordWrap($body);
                 $body = '';
-                $strip_sigs = getPref($data_dir, $username, 'strip_sigs');
-                foreach ($rewrap_body as $line) {
-                    if ($strip_sigs && substr($line,0,3) == '-- ') {
-                        break;
-                    }
-                    if (preg_match("/^(>+)/", $line, $matches)) {
+                $cnt = count($rewrap_body);
+                for ($i=0;$i<$cnt;$i++) {
+                    sqWordWrap($rewrap_body[$i], $editor_size, $default_charset);
+                    if (preg_match("/^(>+)/", $rewrap_body[$i], $matches)) {
                         $gt = $matches[1];
-                        $body .= $body_quote . str_replace("\n", "\n$body_quote$gt ", rtrim($line)) ."\n";
+                        $body .= $body_quote . str_replace("\n", "\n" . $body_quote
+                              . "$gt ", rtrim($rewrap_body[$i])) ."\n";
                     } else {
-                        $body .= $body_quote . (!empty($body_quote) ? ' ' : '') . str_replace("\n", "\n$body_quote" . (!empty($body_quote) ? ' ' : ''), rtrim($line)) . "\n";
+                        $body .= $body_quote . (!empty($body_quote) ? ' ' : '') . str_replace("\n", "\n" . $body_quote . (!empty($body_quote) ? ' ' : ''), rtrim($rewrap_body[$i])) . "\n";
                     }
+                    unset($rewrap_body[$i]);
                 }
-
-                //rewrap the body to clean up quotations and line lengths
-                $body = sqBodyWrap ($body, $editor_size);
-
                 $body = getReplyCitation($from , $orig_header->date) . $body;
                 $composeMessage->reply_rfc822_header = $orig_header;
 
@@ -252,9 +248,10 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
             default:
                 break;
         }
-/// CHANGE FOR SPAM BUTTONS PLUGIN
+/// CHANGE FOR SPAM_BUTTONS PLUGIN
 ///        $compose_messages[$session] = $composeMessage;
-///        sqsession_register($compose_messages, 'compose_messages');
+///        // Not used any more, but left for posterity
+///        //sqsession_register($compose_messages, 'compose_messages');
 ///        session_write_close();
 ///        sqimap_logout($imapConnection);
     }
@@ -269,29 +266,53 @@ function newMail ($mailbox='', $passed_id='', $passed_ent_id='', $action='', $se
     return ($ret);
 } /* function newMail() */
 
+function getforwardHeader($orig_header) {
+    global $editor_size;
 
-function getforwardSubject($subject)
-{
-    if ((substr(strtolower($subject), 0, 4) != 'fwd:') &&
-            (substr(strtolower($subject), 0, 5) != '[fwd:') &&
-            (substr(strtolower($subject), 0, 6) != '[ fwd:')) {
-        $subject = '[Fwd: ' . $subject . ']';
+    $display = array( _("Subject") => strlen(_("Subject")),
+            _("From")    => strlen(_("From")),
+            _("Date")    => strlen(_("Date")),
+            _("To")      => strlen(_("To")),
+            _("Cc")      => strlen(_("Cc")) );
+    $maxsize = max($display);
+    $indent = str_pad('',$maxsize+2);
+    foreach($display as $key => $val) {
+        $display[$key] = $key .': '. str_pad('', $maxsize - $val);
     }
-    return $subject;
+    $from = decodeHeader($orig_header->getAddr_s('from',"\n$indent"),false,false,true);
+    $from = str_replace('&nbsp;',' ',$from);
+    $to = decodeHeader($orig_header->getAddr_s('to',"\n$indent"),false,false,true);
+    $to = str_replace('&nbsp;',' ',$to);
+    $subject = decodeHeader($orig_header->subject,false,false,true);
+    $subject = str_replace('&nbsp;',' ',$subject);
+    $bodyTop =  str_pad(' '._("Original Message").' ',$editor_size -2,'-',STR_PAD_BOTH) .
+        "\n". $display[_("Subject")] . $subject . "\n" .
+        $display[_("From")] . $from . "\n" .
+        $display[_("Date")] . getLongDateString( $orig_header->date, $orig_header->date_unparsed ). "\n" .
+        $display[_("To")] . $to . "\n";
+    if ($orig_header->cc != array() && $orig_header->cc !='') {
+        $cc = decodeHeader($orig_header->getAddr_s('cc',"\n$indent"),false,false,true);
+        $cc = str_replace('&nbsp;',' ',$cc);
+        $bodyTop .= $display[_("Cc")] .$cc . "\n";
+    }
+    $bodyTop .= str_pad('', $editor_size -2 , '-') .
+        "\n\n";
+    return $bodyTop;
 }
 
-
 function getMessage_RFC822_Attachment($message, $composeMessage, $passed_id,
         $passed_ent_id='', $imapConnection) {
+    global $attachment_dir, $username, $data_dir, $uid_support;
+    $hashed_attachment_dir = getHashedDir($username, $attachment_dir);
     if (!$passed_ent_id) {
         $body_a = sqimap_run_command($imapConnection,
                 'FETCH '.$passed_id.' RFC822',
                 TRUE, $response, $readmessage,
-                TRUE);
+                $uid_support);
     } else {
         $body_a = sqimap_run_command($imapConnection,
                 'FETCH '.$passed_id.' BODY['.$passed_ent_id.']',
-                TRUE, $response, $readmessage, TRUE);
+                TRUE, $response, $readmessage, $uid_support);
         $message = $message->parent;
     }
     if ($response == 'OK') {
@@ -300,10 +321,10 @@ function getMessage_RFC822_Attachment($message, $composeMessage, $passed_id,
         array_pop($body_a);
         $body = implode('', $body_a) . "\r\n";
 
-        global $username, $attachment_dir;
-        $hashed_attachment_dir = getHashedDir($username, $attachment_dir);
-        $localfilename = sq_get_attach_tempfile();
-        $fp = fopen($hashed_attachment_dir . '/' . $localfilename, 'wb');
+        $localfilename = GenerateRandomString(32, 'FILE', 7);
+        $full_localfilename = "$hashed_attachment_dir/$localfilename";
+
+        $fp = fopen($full_localfilename, 'w');
         fwrite ($fp, $body);
         fclose($fp);
         $composeMessage->initAttachment('message/rfc822',$subject.'.msg',
@@ -312,7 +333,6 @@ function getMessage_RFC822_Attachment($message, $composeMessage, $passed_id,
     return $composeMessage;
 }
 
-
 /**
  * temporary function to make use of the deliver class.
  * In the future the responsible backend should be automaticly loaded
@@ -321,10 +341,10 @@ function getMessage_RFC822_Attachment($message, $composeMessage, $passed_id,
  */
 function deliverMessage($composeMessage, $draft=false) {
     global $send_to, $send_to_cc, $send_to_bcc, $mailprio, $subject, $body,
-        $username, $identity, $idents, $data_dir,
-        $request_mdn, $request_dr, $default_charset, $useSendmail,
-        $domain, $action, $default_move_to_sent, $move_to_sent,
-        $imapServerAddress, $imapPort, $sent_folder, $key;
+        $username, $popuser, $usernamedata, $identity, $idents, $data_dir,
+        $request_mdn, $request_dr, $default_charset, $color, $useSendmail,
+        $domain, $action, $default_move_to_sent, $move_to_sent;
+    global $imapServerAddress, $imapPort, $sent_folder, $key;
 /// CHANGE FOR SPAM_BUTTONS PLUGIN
 /* --- do we need to do overrides again here?  should not need to, but
 //     someone reported problems with the default not being overriden
@@ -372,26 +392,43 @@ function deliverMessage($composeMessage, $draft=false) {
     }
     $composeMessage->setBody($body);
 
+    if (ereg("^([^@%/]+)[@%/](.+)$", $username, $usernamedata)) {
+        $popuser = $usernamedata[1];
+        $domain  = $usernamedata[2];
+        unset($usernamedata);
+    } else {
+        $popuser = $username;
+    }
     $reply_to = '';
+    $from_mail = $idents[$identity]['email_address'];
+    $full_name = $idents[$identity]['full_name'];
     $reply_to  = $idents[$identity]['reply_to'];
-
-    $from_addr = build_from_header($identity);
-    $rfc822_header->from = $rfc822_header->parseAddress($from_addr,true);
+    if (!$from_mail) {
+        $from_mail = "$popuser@$domain";
+    }
+    $rfc822_header->from = $rfc822_header->parseAddress($from_mail,true);
+    if ($full_name) {
+        $from = $rfc822_header->from[0];
+        if (!$from->host) $from->host = $domain;
+        $full_name_encoded = encodeHeader($full_name);
+        if ($full_name_encoded != $full_name) {
+            $from_addr = $full_name_encoded .' <'.$from->mailbox.'@'.$from->host.'>';
+        } else {
+            $from_addr = '"'.$full_name .'" <'.$from->mailbox.'@'.$from->host.'>';
+        }
+        $rfc822_header->from = $rfc822_header->parseAddress($from_addr,true);
+    }
     if ($reply_to) {
         $rfc822_header->reply_to = $rfc822_header->parseAddress($reply_to,true);
     }
     /* Receipt: On Read */
     if (isset($request_mdn) && $request_mdn) {
-        $rfc822_header->dnt = $rfc822_header->parseAddress($from_addr,true);
-    } elseif (isset($rfc822_header->dnt)) {
-        unset($rfc822_header->dnt);
+        $rfc822_header->dnt = $rfc822_header->parseAddress($from_mail,true);
     }
 
     /* Receipt: On Delivery */
     if (isset($request_dr) && $request_dr) {
-        $rfc822_header->more_headers['Return-Receipt-To'] = $from->mailbox.'@'.$from->domain;
-    } elseif (isset($rfc822_header->more_headers['Return-Receipt-To'])) {
-        unset($rfc822_header->more_headers['Return-Receipt-To']);
+        $rfc822_header->more_headers['Return-Receipt-To'] = $from_mail;
     }
 
     /* multipart messages */
@@ -429,18 +466,26 @@ function deliverMessage($composeMessage, $draft=false) {
     $composeMessage->rfc822_header = $rfc822_header;
 
     /* Here you can modify the message structure just before we hand
-       it over to deliver; plugin authors note that $composeMessage
-       is sent and modified by reference since 1.5.2 */
-/// CHANGE FOR SPAM BUTTONS PLUGIN
-///    do_hook('compose_send', $composeMessage);
+       it over to deliver */
+/// CHANGE FOR SPAM_BUTTONS PLUGIN
+///    $hookReturn = do_hook('compose_send', $composeMessage);
+///    /* Get any changes made by plugins to $composeMessage. */
+///    if ( is_object($hookReturn[1]) ) {
+///        $composeMessage = $hookReturn[1];
+///    }
 
     if (!$useSendmail && !$draft) {
         require_once(SM_PATH . 'class/deliver/Deliver_SMTP.class.php');
         $deliver = new Deliver_SMTP();
-        global $smtpServerAddress, $smtpPort, $pop_before_smtp;
+        global $smtpServerAddress, $smtpPort, $pop_before_smtp, $smtp_auth_mech;
 
         $authPop = (isset($pop_before_smtp) && $pop_before_smtp) ? true : false;
+
+        $user = '';
+        $pass = '';
+
         get_smtp_user($user, $pass);
+
         $stream = $deliver->initStream($composeMessage,$domain,0,
                 $smtpServerAddress, $smtpPort, $user, $pass, $authPop);
     } elseif (!$draft) {
@@ -459,7 +504,7 @@ function deliverMessage($composeMessage, $draft=false) {
     } elseif ($draft) {
         global $draft_folder;
         require_once(SM_PATH . 'class/deliver/Deliver_IMAP.class.php');
-        $imap_stream = sqimap_login($username, false, $imapServerAddress,
+        $imap_stream = sqimap_login($username, $key, $imapServerAddress,
                 $imapPort, 0);
         if (sqimap_mailbox_exists ($imap_stream, $draft_folder)) {
             require_once(SM_PATH . 'class/deliver/Deliver_IMAP.class.php');
@@ -473,35 +518,27 @@ function deliverMessage($composeMessage, $draft=false) {
             $composeMessage->purgeAttachments();
             return $length;
         } else {
-//SPAM BUTTONS NOTE: htmlspecialchars is not applied to $draft_folder below in anticipation of the core using sq_htmlspecialchars when strings are assinged to the template
-            $msg  = "\n" 
-                  . sprintf(_("Error: Draft folder %s does not exist."), $draft_folder);
-            plain_error_message($msg);
+            $msg  = '<br />'.sprintf(_("Error: Draft folder %s does not exist."),
+                htmlspecialchars($draft_folder));
+            plain_error_message($msg, $color);
             return false;
         }
     }
-    $success = false;
+    $succes = false;
     if ($stream) {
         $length = $deliver->mail($composeMessage, $stream);
-        $success = $deliver->finalizeStream($stream);
+        $succes = $deliver->finalizeStream($stream);
     }
-    if (!$success) {
-        // $deliver->dlv_server_msg is not always server's reply
-        $msg = _("Message not sent.") . "\n" .
-            $deliver->dlv_msg;
-        if (!empty($deliver->dlv_server_msg)) {
-            // add 'server replied' part only when it is not empty.
-            // Delivery error can be generated by delivery class itself
-            $msg .= "\n" 
-                 . _("Server replied:") . ' ' . $deliver->dlv_ret_nr . ' ' .
-//SPAM BUTTONS NOTE: htmlspecialchars is not applied to $deliver->dlv_server_msg below in anticipation of the core using sq_htmlspecialchars when strings are assinged to the template
-                nl2br($deliver->dlv_server_msg);
-        }
-        plain_error_message($msg);
+    if (!$succes) {
+        $msg  = _("Message not sent.") .' '.  _("Server replied:") .
+            "\n<blockquote>\n" . $deliver->dlv_msg . '<br />' .
+            $deliver->dlv_ret_nr . ' ' .
+            $deliver->dlv_server_msg . "</blockquote>\n\n";
+        plain_error_message($msg, $color);
     } else {
         unset ($deliver);
         $move_to_sent = getPref($data_dir,$username,'move_to_sent');
-        $imap_stream = sqimap_login($username, false, $imapServerAddress, $imapPort, 0);
+        $imap_stream = sqimap_login($username, $key, $imapServerAddress, $imapPort, 0);
 
         /* Move to sent code */
         if (isset($default_move_to_sent) && ($default_move_to_sent != 0)) {
@@ -523,14 +560,7 @@ function deliverMessage($composeMessage, $draft=false) {
             $lcl_allow_sent = false;
         }
 
-        global $passed_id, $mailbox;
         if (($fld_sent && $svr_allow_sent && !$lcl_allow_sent) || ($fld_sent && $lcl_allow_sent)) {
-            if ($action == 'reply' || $action == 'reply_all') {
-                $save_reply_with_orig=getPref($data_dir,$username,'save_reply_with_orig');
-                if ($save_reply_with_orig) {
-                    $sent_folder = $mailbox;
-                }
-            }
             sqimap_append ($imap_stream, $sent_folder, $length);
             require_once(SM_PATH . 'class/deliver/Deliver_IMAP.class.php');
             $imap_deliver = new Deliver_IMAP();
@@ -538,58 +568,104 @@ function deliverMessage($composeMessage, $draft=false) {
             sqimap_append_done ($imap_stream, $sent_folder);
             unset ($imap_deliver);
         }
-
-        global $what, $iAccount, $startMessage;
-
+        global $passed_id, $mailbox, $action;
         $composeMessage->purgeAttachments();
-        if ($action=='reply' || $action=='reply_all' || $action=='forward' || $action=='forward_as_attachment') {
-            require(SM_PATH . 'functions/mailbox_display.php');
-            $aMailbox = sqm_api_mailbox_select($imap_stream, $iAccount, $mailbox,array('setindex' => $what, 'offset' => $startMessage),array());
-            switch($action) {
-            case 'reply':
-            case 'reply_all':
-                // check if we are allowed to set the \\Answered flag
-                if (in_array('\\answered',$aMailbox['PERMANENTFLAGS'], true)) {
-                    $aUpdatedMsgs = sqimap_toggle_flag($imap_stream, array($passed_id), '\\Answered', true, false);
-                    if (isset($aUpdatedMsgs[$passed_id]['FLAGS'])) {
-                        /**
-                        * Only update the cached headers if the header is
-                        * cached.
-                        */
-                        if (isset($aMailbox['MSG_HEADERS'][$passed_id])) {
-                            $aMailbox['MSG_HEADERS'][$passed_id]['FLAGS'] = $aMsg['FLAGS'];
-                        }
-                    }
-                }
-                break;
-            case 'forward':
-            case 'forward_as_attachment':
-                // check if we are allowed to set the $Forwarded flag (RFC 4550 paragraph 2.8)
-                if (in_array('$forwarded',$aMailbox['PERMANENTFLAGS'], true) ||
-                    in_array('\\*',$aMailbox['PERMANENTFLAGS'])) {
-
-                    $aUpdatedMsgs = sqimap_toggle_flag($imap_stream, array($passed_id), '$Forwarded', true, false);
-                    if (isset($aUpdatedMsgs[$passed_id]['FLAGS'])) {
-                        if (isset($aMailbox['MSG_HEADERS'][$passed_id])) {
-                            $aMailbox['MSG_HEADERS'][$passed_id]['FLAGS'] = $aMsg['FLAGS'];
-                        }
-                    }
-                }
-                break;
-            }
-
-            /**
-             * Write mailbox with updated seen flag information back to cache.
-             */
-            if(isset($aUpdatedMsgs[$passed_id])) {
-                $mailbox_cache[$iAccount.'_'.$aMailbox['NAME']] = $aMailbox;
-                sqsession_register($mailbox_cache,'mailbox_cache');
-            }
-
-            sqimap_logout($imap_stream);
+        if ($action == 'reply' || $action == 'reply_all') {
+            sqimap_mailbox_select ($imap_stream, $mailbox);
+            sqimap_messages_flag ($imap_stream, $passed_id, $passed_id, 'Answered', false);
         }
+        sqimap_logout($imap_stream);
     }
-    return $success;
+    return $succes;
 }
 
 
+
+if (!function_exists('is_conversion_safe'))
+{
+/**
+ * Function informs if it is safe to convert given charset to the one that is used by user.
+ *
+ * It is safe to use conversion only if user uses utf-8 encoding and when
+ * converted charset is similar to the one that is used by user.
+ *
+ * @param string $input_charset Charset of text that needs to be converted
+ * @return bool is it possible to convert to user's charset
+ */
+function is_conversion_safe($input_charset) {
+    global $languages, $sm_notAlias, $default_charset, $lossy_encoding;
+
+    if (isset($lossy_encoding) && $lossy_encoding )
+        return true;
+
+    // convert to lower case
+    $input_charset = strtolower($input_charset);
+
+    // Is user's locale Unicode based ?
+    if ( $default_charset == "utf-8" ) {
+        return true;
+    }
+
+    // Charsets that are similar
+    switch ($default_charset) {
+    case "windows-1251":
+        if ( $input_charset == "iso-8859-5" ||
+           $input_charset == "koi8-r" ||
+           $input_charset == "koi8-u" ) {
+        return true;
+        } else {
+            return false;
+        }
+    case "windows-1257":
+        if ( $input_charset == "iso-8859-13" ||
+             $input_charset == "iso-8859-4" ) {
+            return true;
+        } else {
+            return false;
+        }
+    case "iso-8859-4":
+        if ( $input_charset == "iso-8859-13" ||
+             $input_charset == "windows-1257" ) {
+            return true;
+        } else {
+            return false;
+        }
+    case "iso-8859-5":
+        if ( $input_charset == "windows-1251" ||
+             $input_charset == "koi8-r" ||
+             $input_charset == "koi8-u" ) {
+            return true;
+        } else {
+            return false;
+        }
+    case "iso-8859-13":
+        if ( $input_charset == "iso-8859-4" ||
+             $input_charset == "windows-1257" ) {
+            return true;
+        } else {
+            return false;
+        }
+    case "koi8-r":
+        if ( $input_charset == "windows-1251" ||
+             $input_charset == "iso-8859-5" ||
+             $input_charset == "koi8-u" ) {
+            return true;
+        } else {
+            return false;
+        }
+    case "koi8-u":
+        if ( $input_charset == "windows-1251" ||
+             $input_charset == "iso-8859-5" ||
+             $input_charset == "koi8-r" ) {
+            return true;
+        } else {
+            return false;
+        }
+    default:
+        return false;
+    }
+}
+}
+
+
+
