@@ -1,5 +1,18 @@
 <?php
 
+/**
+  * SquirrelMail Spam Buttons Plugin
+  * Copyright (c) 2005-2008 Paul Lesniewski <paul@squirrelmail.org>,
+  * Licensed under the GNU GPL. For full terms see the file COPYING.
+  *
+  * @package plugins
+  * @subpackage spam_buttons
+  *
+  * See below, starting with "GENERAL OPTIONS" for the beginning 
+  * of the configuration section.
+  *
+  */
+
 global $show_spam_buttons_on_read_body, $show_spam_buttons_on_message_list,
        $show_spam_link_on_read_body, $is_spam_shell_command, $is_spam_resend_destination,
        $is_not_spam_shell_command, $is_not_spam_resend_destination,
@@ -20,7 +33,11 @@ global $show_spam_buttons_on_read_body, $show_spam_buttons_on_message_list,
        $sb_suppress_not_spam_button_folder_allow_override,
        $sb_spam_header_name, $sb_not_spam_header_name,
        $sb_spam_header_value, $sb_not_spam_header_value,
-       $reported_spam_text, $reported_not_spam_text;
+       $sb_show_spam_button_folder, $sb_show_spam_button_folder_allow_override,
+       $sb_show_not_spam_button_folder, $sb_show_not_spam_button_folder_allow_override,
+       $reported_spam_text, $reported_not_spam_text,
+       $sb_move_to_other_message_after_report, $sb_auto_create_destination_folder,
+       $sb_move_to_other_message_after_report_allow_override;
 
 
 
@@ -84,6 +101,51 @@ $show_spam_buttons_on_read_body = 1;
 
 
 
+// When viewing an individual message, this plugin can determine if
+// it was tagged as spam or ham by your anti-spam scanner if a certain
+// header ($sb_spam_header_name) is added to the message.  By comparing 
+// the value of that header to what is configured here 
+// ($sb_spam_header_value), this plugin can display only the HAM (not 
+// spam) button on messages marked as spam, or the SPAM button on messages 
+// marked as ham (not spam), or both buttons otherwise.
+//
+// When in use, this functionality will override 
+// $sb_suppress_spam_button_folder, $sb_suppress_not_spam_button_folder
+// $sb_show_spam_button_folder and $sb_show_not_spam_button_folder
+// (buttons/links displayed in the read-message screen will be based on
+// the message headers and NOT what folder the message is in).
+//
+// Note that $sb_spam_header_name and $sb_not_spam_header_name are usually 
+// the same thing in most spam scanner systems.
+//
+// The values for $sb_spam_header_value and $sb_not_spam_header_value
+// are regular expressions that will be used to compare the actual 
+// message header values.
+//
+// $sb_spam_header_name = 'X-Spam-Status';
+// $sb_spam_header_value = '/^Yes/i';
+// $sb_not_spam_header_name = 'X-Spam-Status';
+// $sb_not_spam_header_value = '/^No/i';
+//
+$sb_spam_header_name = '';
+$sb_spam_header_value = '';
+$sb_not_spam_header_name = '';
+$sb_not_spam_header_value = '';
+
+
+
+// When any one of the report-by-move/copy or move/copy-after-report
+// options is set to a folder that does not exist for any one user,
+// should it be created automatically?  If not, an error will occur
+// when the user reports spam/ham.
+//
+// 0 = don't create folders automatically
+// 1 = create folders if they do not exist upon spam/ham report
+//
+$sb_auto_create_destination_folder = 0;
+
+
+
 // Debugging: Turning this on will dump out the command text, the
 //            message body and the results of the spam report if
 //            done using a shell command.
@@ -107,15 +169,20 @@ $sb_debug = 0;
 //
 
 
-// When the user is in this folder, suppress all SPAM button/links
+// When the user is in this folder (or folders), suppress
+// all SPAM button/links
 //
-// Set to an empty string to show the spam button in all folders, 
-// or the exact name of the specified spam folder (the format of 
-// which may depend on your IMAP server).
+// Set to an empty list to show the spam button in all folders, 
+// or the exact name of the specified spam folder(s) (the format
+// of which may depend on your IMAP server).
 //
-// $sb_suppress_spam_button_folder = 'INBOX.Spam';
+// In the case of collision with $sb_show_spam_button_folder 
+// (see below), this setting will lose.
 //
-$sb_suppress_spam_button_folder = '';
+// $sb_suppress_spam_button_folder = array('INBOX.Spam');
+// $sb_suppress_spam_button_folder = array('INBOX.Spam', 'INBOX.Junk');
+//
+$sb_suppress_spam_button_folder = array();
 
 
 
@@ -128,15 +195,20 @@ $sb_suppress_spam_button_folder_allow_override = 1;
 
 
 
-// When the user is in this folder, suppress all HAM (not spam) button/links
+// When the user is in this folder (or folders), suppress 
+// all HAM (not spam) button/links
 //
-// Set to an empty string to show the ham button in all folders,
-// or the exact name of the specified ham folder (the format of 
-// which may depend on your IMAP server).
+// Set to an empty list to show the ham button in all folders,
+// or the exact name of the specified ham folder(s) (the format
+// of which may depend on your IMAP server).
 //
-// $sb_suppress_not_spam_button_folder = 'INBOX.Ham';
+// In the case of collision with $sb_show_not_spam_button_folder 
+// (see below), this setting will lose.
 //
-$sb_suppress_not_spam_button_folder = '';
+// $sb_suppress_not_spam_button_folder = array('INBOX.Ham');
+// $sb_suppress_not_spam_button_folder = array('INBOX.Ham', 'INBOX.Personal');
+//
+$sb_suppress_not_spam_button_folder = array();
 
 
 
@@ -145,39 +217,73 @@ $sb_suppress_not_spam_button_folder = '';
 // if so, or set to 0 (zero) if the administrator's 
 // setting above goes for all users.
 // 
-$sb_suppress_not_spam_button_folder_allow_override = 1;
+$sb_suppress_not_spam_button_folder_allow_override = 0;
 
 
 
-// When viewing an individual message, this plugin can determine if
-// it was tagged as spam or ham by your anti-spam scanner if a certain
-// header ($sb_spam_header_name) is added to the message.  By comparing 
-// the value of that header to what is configured here 
-// ($sb_spam_header_value), this plugin can display only the HAM (not 
-// spam) button on messages marked as spam, or the SPAM button on messages 
-// marked as ham (not spam), or both buttons otherwise.
+// -------------------------------------------------------------------
 //
-// When in use, this functionality will override 
-// $sb_suppress_spam_button_folder and $sb_suppress_not_spam_button_folder
-// (buttons/links displayed in the read-message screen will be based on
-// the message headers and NOT what folder the message is in).
+// BUTTON/LINK INCLUSION OPTIONS
 //
-// Note that $sb_spam_header_name and $sb_not_spam_header_name are usually 
-// the same thing in most spam scanner systems.
+
+
+// Only when the user is in this folder (or folders), 
+// show SPAM button/links
 //
-// The values for $sb_spam_header_value and $sb_not_spam_header_value
-// are regular expressions that will be used to compare the actual 
-// message header values.
+// When this option is set to anything other than an
+// empty list, spam buttons/links won't be shown in
+// any folder other than the one(s) specified here.
 //
-// $sb_spam_header_name = 'X-Spam-Status';
-// $sb_spam_header_value = '/^Yes/i';
-// $sb_not_spam_header_name = 'X-Spam-Status';
-// $sb_not_spam_header_value = '/^No/i';
+// In the case of collision with 
+// $sb_suppress_spam_button_folder, this setting will win.
 //
-$sb_spam_header_name = '';
-$sb_spam_header_value = '';
-$sb_not_spam_header_name = '';
-$sb_not_spam_header_value = '';
+// If set, this value must contain the exact name(s) 
+// of the specified folder(s) (the format of which 
+// may depend on your IMAP server).
+//
+// $sb_show_spam_button_folder = array('INBOX.Ham');
+// $sb_show_spam_button_folder = array('INBOX.Ham', 'INBOX.Personal');
+//
+$sb_show_spam_button_folder = array();
+
+
+
+// Should users be able to choose what the value
+// of $sb_show_spam_button_folder is?  Set to 1
+// if so, or set to 0 (zero) if the administrator's
+// setting above goes for all users.
+//
+$sb_show_spam_button_folder_allow_override = 0;
+
+
+
+// Only when the user is in this folder (or folders), 
+// show HAM (not spam) button/links
+//
+// When this option is set to anything other than an
+// empty list, ham buttons/links won't be shown in
+// any folder other than the one(s) specified here.
+//
+// In the case of collision with 
+// $sb_suppress_not_spam_button_folder, this setting will win.
+//
+// If set, this value must contain the exact name(s) 
+// of the specified folder(s) (the format of which 
+// may depend on your IMAP server).
+//
+// $sb_show_not_spam_button_folder = array('INBOX.Spam');
+// $sb_show_not_spam_button_folder = array('INBOX.Spam', 'INBOX.Junk');
+//
+$sb_show_not_spam_button_folder = array();
+
+
+
+// Should users be able to choose what the value
+// of $sb_show_not_spam_button_folder is?  Set to 1
+// if so, or set to 0 (zero) if the administrator's
+// setting above goes for all users.
+//
+$sb_show_not_spam_button_folder_allow_override = 1;
 
 
 
@@ -325,6 +431,32 @@ $sb_reselect_messages_allow_override = 1;
 
 
 
+// When either report-by-move or delete-after-report is
+// enabled, should the user be directed to the next or
+// previous message after the report, or just back to
+// the message list?
+//
+// Acceptable values are "next", "previous" and "" (empty),
+// which indicates that the user will be returned to the
+// message list.
+//
+// $sb_move_to_other_message_after_report = 'next';
+// $sb_move_to_other_message_after_report = 'previous';
+// $sb_move_to_other_message_after_report = '';
+//
+$sb_move_to_other_message_after_report = '';
+
+
+
+// Should users be able to choose what the value
+// of $sb_move_to_other_message_after_report is?  
+// Set to 1 if so, or set to 0 (zero) if the 
+// administrator's setting above goes for all users.
+//
+$sb_move_to_other_message_after_report_allow_override = 1;
+
+
+
 // -------------------------------------------------------------------
 //
 // REPORT BY MOVE-TO-FOLDER OPTIONS
