@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Spam Buttons Plugin
-  * Copyright (c) 2005-2007 Paul Lesniewski <paul@squirrelmail.org>,
+  * Copyright (c) 2005-2008 Paul Lesniewski <paul@squirrelmail.org>,
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
   * @package plugins
@@ -79,6 +79,12 @@ function squirrelmail_plugin_init_spam_buttons()
       = 'sb_button_action';
 
 
+   // SM 1.5.x - abort message view when message is moved/deleted
+   //
+   $squirrelmail_plugin_hooks['template_construct_read_message_body.tpl']['spam_buttons']
+      = 'sb_abort_message_view';
+
+
    // show options on display preferences page
    //
    $squirrelmail_plugin_hooks['optpage_loadhook_display']['spam_buttons']
@@ -126,7 +132,7 @@ function spam_buttons_info()
                        'sm_site_username' => 'pdontthink',
                     ),
                  ),
-                 'version' => '2.0',
+                 'version' => '2.1',
                  'required_sm_version' => '1.4.0',
                  'requires_configuration' => 1,
                  'summary' => 'Puts Spam/Not Spam buttons on mailbox list and message view pages.',
@@ -173,7 +179,7 @@ function spam_buttons_version()
   */
 function sb_button_action($args)
 {
-   include_once(SM_PATH . 'plugins/spam_buttons/functions.php');
+   include_once(SM_PATH . 'plugins/spam_buttons/report.php');
    return sb_button_action_do($args);
 }
 
@@ -185,7 +191,7 @@ function sb_button_action($args)
   */
 function sb_mailbox_list_buttons(&$args)
 {
-   include_once(SM_PATH . 'plugins/spam_buttons/functions.php');
+   include_once(SM_PATH . 'plugins/spam_buttons/buttons.php');
    sb_mailbox_list_buttons_do($args);
 }
 
@@ -197,7 +203,7 @@ function sb_mailbox_list_buttons(&$args)
   */
 function sb_read_message_buttons($args)
 {
-   include_once(SM_PATH . 'plugins/spam_buttons/functions.php');
+   include_once(SM_PATH . 'plugins/spam_buttons/buttons.php');
    return sb_read_message_buttons_do($args);
 }
 
@@ -209,7 +215,7 @@ function sb_read_message_buttons($args)
   */
 function sb_read_message_links(&$links)
 {
-   include_once(SM_PATH . 'plugins/spam_buttons/functions.php');
+   include_once(SM_PATH . 'plugins/spam_buttons/buttons.php');
    sb_read_message_links_do($links);
 }
 
@@ -221,7 +227,7 @@ function sb_read_message_links(&$links)
   */
 function spam_buttons_display_options()
 {
-   include_once(SM_PATH . 'plugins/spam_buttons/functions.php');
+   include_once(SM_PATH . 'plugins/spam_buttons/options.php');
    spam_buttons_display_options_do();
 }
 
@@ -236,9 +242,22 @@ function spam_buttons_display_options()
   */
 function spam_buttons_check_configuration()
 {
-   include_once(SM_PATH . 'plugins/spam_buttons/functions.php');
+   include_once(SM_PATH . 'plugins/spam_buttons/configtest.php');
    return spam_buttons_check_configuration_do();
 }
 
 
 
+/**
+  * Abort message view when message is moved/deleted
+  * out from under current message view
+  *
+  */
+function sb_abort_message_view()
+{
+   include_once(SM_PATH . 'plugins/spam_buttons/report.php');
+   sb_abort_message_view_do();
+}
+
+
+
